package main

import (
	"flag"
	"fmt"
	"os"
	"runtime/debug"
	"strings"

	"baal.ar76.eu/x/pub/multisql/internal/cfg"
	"baal.ar76.eu/x/pub/multisql/internal/mgr"
	"baal.ar76.eu/x/pub/multisql/internal/pass"

	"log"
)

func main() {
	log.SetFlags(log.LstdFlags)
	params := cfg.GetParams()
	if params.Version {
		printVersion()
		return
	}
	args := flag.Args()
	if len (args) > 0 {
		encryption(args, params)
		return
	}

	if params.LogFile != "" {
		fh, err := os.OpenFile(params.LogFile, os.O_CREATE|os.O_WRONLY|os.O_APPEND, 0o664)
		if err != nil {
			log.Fatalf("Nie udało się ustawić logu: %v", err)
		}
		defer fh.Close()
		log.SetOutput(fh)
	}

	if params.Verbose {
		log.Print("MultiSQL - start")
	}
	config, err := cfg.GetConfig(params)

	if err != nil {
		log.Fatalf("Błąd z konfiguracją: %v\n", err)
	}


	if params.OutDir == "" {
		params.OutDir = "."
	} else {
		fs, err := os.Stat(params.OutDir)
		if err != nil {
			log.Fatalf("Podany katalog wyników nie jest prawidłowy: %v\n", err)
		}
		if !fs.IsDir() {
			log.Fatalf("Ścieżka %s nie wskazuje na poprawny katalog\n", params.OutDir)
		}
	}

	manager, err := mgr.Create(params.Verbose, params.AskPass, params.OutDir, config)
	if err != nil {
		log.Fatalf("Nie można wystartować operacji: %v\n", err)
	}

	err = manager.Run()

	if err != nil {
		log.Fatalf("Błąd przetwarzania: %v\n", err)
	}

}

func encryption(args []string, params cfg.Parameters) {
	switch strings.ToLower(args[0]) {
	case "encrypt":
		if len(args) < 3 {
			log.Fatal("encrypt wymaga dwu plików: cel-plik.zaszyfrowany  źródło-plik.jawny")
			return
		}
		plain, err := os.ReadFile(args[2])
		if err != nil {
			log.Fatalf("Błąd odczytu pliku: %v", err)
		}

		password := pass.EnterMasterPass()

		err = pass.EncryptFile(password, args[1], plain)
		if err != nil {
			_ = os.Remove(args[1]) // jeśli błąd to usuń plik
			log.Fatalf("Błąd szyfrowania pliku: %v", err)
		}
	case "decrypt":
		if len(args) < 2 {
			log.Fatal("decryp wymaga podania pliku zaszyfrowanego")
			return
		}
		password := pass.GetMasterPass(params.AskPass)

		data, err := pass.DecryptFile(password, args[1])
		if err !=nil {
			log.Fatalf("Błąd deszyfrowania pliku: %v", err)
		}
		if len(args) > 2 {
			err = os.WriteFile(args[2], data, 0o600)
			if err != nil {
				log.Fatalf("Błąd zapisu pliku: %v", err)
			}
			log.Printf("Zapisano odszyfrowane dane do pliku %s", args[2])
		} else {
			os.Stdout.Write(data)
		}

	default:
		log.Println("W trybie komend wymagane jest podanie polecenie encrypt lub decrypt i nazw plików: zaszyfrowanego i jawnego");
		log.Println("Np:")
		log.Println("multisql encrypt hasła.zaszyfrowany hasła")
		log.Println("multisql decrypt hasła.zaszyfrowany hasła")
	}
}

func printVersion() {
	bi, ok := debug.ReadBuildInfo()
	if !ok {
		log.Fatalf("Błąd odczytu informacji o wersji")
	}
	fmt.Fprintf(os.Stderr, "Wersja głównego modułu: %v\n", bi.Main.Version)
	fmt.Fprintln(os.Stderr, bi)
}
